@@ -21,15 +21,13 @@ from enum import Enum
 from functools import cached_property
 from typing import TYPE_CHECKING, NewType
 
+from data_platform_helpers.advanced_statuses.models import StatusObject
+from data_platform_helpers.advanced_statuses.protocol import ManagerStatusProtocol
+from data_platform_helpers.advanced_statuses.types import Scope
 from ops import Container
 from ops.framework import Object
 from ops.model import (
-    ActiveStatus,
-    BlockedStatus,
-    MaintenanceStatus,
     Relation,
-    StatusBase,
-    WaitingStatus,
 )
 from tenacity import (
     Retrying,
@@ -46,8 +44,8 @@ from single_kernel_mongo.config.literals import (
     Substrates,
     TrustStoreFiles,
 )
-from single_kernel_mongo.config.models import CharmSpec
-from single_kernel_mongo.core.status_provider import StatusProvider
+from single_kernel_mongo.config.models import BackupState, CharmSpec
+from single_kernel_mongo.config.statuses import BackupStatuses
 from single_kernel_mongo.core.structured_config import MongoDBRoles
 from single_kernel_mongo.exceptions import (
     BackupError,
@@ -62,11 +60,14 @@ from single_kernel_mongo.exceptions import (
 )
 from single_kernel_mongo.managers.config import BackupConfigManager
 from single_kernel_mongo.state.charm_state import CharmState
+from single_kernel_mongo.state.config_server_state import AppShardingComponentKeys
 from single_kernel_mongo.workload import get_pbm_workload_for_substrate
 from single_kernel_mongo.workload.backup_workload import PBMWorkload
 
 if TYPE_CHECKING:
-    from single_kernel_mongo.managers.mongodb_operator import MongoDBOperator  # pragma: nocover
+    from single_kernel_mongo.managers.mongodb_operator import (
+        MongoDBOperator,
+    )  # pragma: nocover
 
 BackupListType = NewType("BackupListType", list[tuple[str, str, str]])
 
@@ -102,7 +103,7 @@ def _backup_restore_retry_before_sleep(retry_state) -> None:
     )
 
 
-class BackupManager(Object, BackupConfigManager, StatusProvider):
+class BackupManager(Object, BackupConfigManager, ManagerStatusProtocol):
     """Manager for the S3 integrator and backups."""
 
     def __init__(
@@ -113,7 +114,8 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         state: CharmState,
         container: Container | None,
     ) -> None:
-        super().__init__(parent=dependent, key="backup")
+        self.name = "backup"
+        super().__init__(parent=dependent, key=self.name)
         super(Object, self).__init__(
             role=role,
             substrate=substrate,
@@ -128,6 +130,16 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
             role=role, container=container
         )
         self.state = state
+        self._backup_id: str = ""
+
+    @property
+    def backup_id(self) -> str:
+        """The current backup id."""
+        return self._backup_id
+
+    @backup_id.setter
+    def backup_id(self, value: str):
+        self._backup_id = value
 
     @cached_property
     def environment(self) -> dict[str, str]:
@@ -145,12 +157,13 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         """
         return (self.state.s3_relation is None) or (not self.state.is_role(MongoDBRoles.SHARD))
 
-    def on_relation_broken(self, relation: Relation) -> None:
+    def cleanup_certs_and_restart(self, relation: Relation) -> None:
         """On relation broken event, we need to remove the certificate from the trust store."""
         if self.state.is_scaling_down(relation.id):
             logger.info("Relation broken event occurring due to scale down.")
             return
         self.dependent.remove_ca_cert_from_trust_store(TrustStoreFiles.PBM)
+        self.remove_cert_from_shards()
         self.configure_and_restart(force=True)
 
     @retry(
@@ -189,7 +202,6 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
 
     def list_backup_action(self) -> str:
         """List the backups entries."""
-        backup_list: BackupListType = BackupListType([])
         try:
             pbm_status_output = self.pbm_status
         except WorkloadExecError as e:
@@ -281,34 +293,79 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
 
             raise RestoreError(fail_message)
 
-    def get_status(self) -> StatusBase | None:
-        """Gets the PBM status."""
-        if not self.workload.active():
-            return WaitingStatus("waiting for pbm to start")
+    def backup_state(self) -> BackupState:
+        """Gets the backup state that can be mapped to a status."""
+        if not self.state.db_initialised:
+            return BackupState.EMPTY
         if not self.state.s3_relation:
             logger.info("No configuration for backups, not relation to s3-charm")
-            return None
+            return BackupState.EMPTY
         if not self.validate_s3_config():
             logger.info(
                 "Relation to S3 charm exists but not all necessary configurations have been set."
             )
-            return BlockedStatus("s3 configurations missing.")
-        try:
-            previous_status = self.charm.unit.status
-            pbm_status = self.pbm_status
-            pbm_error = self.process_pbm_error(pbm_status)
-            if pbm_error:
-                return BlockedStatus(pbm_error)
+            return BackupState.MISSING_CONFIG
+        if not self.workload.active():
+            return BackupState.WAITING_PBM_START
 
-            processed_status = self.process_pbm_status(pbm_status)
-            operation_result = self._get_backup_restore_operation_result(
-                processed_status, previous_status
-            )
-            logger.info(operation_result)
-            return processed_status
+        try:
+            pbm_status = self.pbm_status
+        except WorkloadExecError as err:
+            pbm_status = err.stdout
+
+        try:
+            if pbm_error := self.process_pbm_error(pbm_status):
+                return pbm_error
+            return self.process_pbm_status(pbm_status)
         except Exception as e:
             logger.error(f"Failed to get pbm status: {e}")
-            return BlockedStatus("PBM error")
+            return BackupState.UNKNOWN_ERROR
+
+    def map_backup_state_to_status(self, state: BackupState) -> list[StatusObject]:  # noqa: C901
+        """Maps the state to a list of statuses."""
+        match state:
+            case BackupState.EMPTY:
+                return []
+            case BackupState.MISSING_CONFIG:
+                return [BackupStatuses.PBM_MISSING_CONFIGS.value]
+            case BackupState.WAITING_PBM_START:
+                return [BackupStatuses.WAITING_FOR_PBM_START.value]
+            case BackupState.INCORRECT_CREDS:
+                return [BackupStatuses.PBM_INCORRECT_CREDS.value]
+            case BackupState.INCOMPATIBLE_CONF:
+                return [BackupStatuses.PBM_INCOMPATIBLE_CONF.value]
+            case BackupState.UNKNOWN_ERROR:
+                return [BackupStatuses.UNKNOWN_PBM_ERROR.value]
+            case BackupState.WAITING_TO_SYNC:
+                return [BackupStatuses.PBM_WAITING_TO_SYNC.value]
+            case BackupState.BACKUP_RUNNING:
+                if operation_result := self._get_backup_restore_operation_result(state):
+                    logger.info(operation_result)
+                return [BackupStatuses.backup_running(self.backup_id)]
+            case BackupState.RESTORE_RUNNING:
+                if operation_result := self._get_backup_restore_operation_result(state):
+                    logger.info(operation_result)
+                return [BackupStatuses.restore_running(self.backup_id)]
+            case BackupState.ACTIVE:
+                return [BackupStatuses.ACTIVE_IDLE.value]
+
+    def get_statuses(self, scope: Scope, recompute: bool = False) -> list[StatusObject]:
+        """Gets the PBM statuses."""
+        if not recompute:
+            return self.state.statuses.get(scope=scope, component=self.name).root
+
+        if not self.state.db_initialised:
+            return []
+
+        if scope == "app":
+            return []
+
+        return self.map_backup_state_to_status(self.backup_state())
+
+    def get_main_status(self) -> StatusObject | None:
+        """Returns the first status of the list."""
+        pbm_statuses = self.get_statuses(scope="unit", recompute=True)
+        return next(iter(pbm_statuses), None)
 
     def resync_config_options(self):  # pragma: nocover
         """Attempts to resync config options and sets status in case of failure."""
@@ -316,6 +373,9 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         self.set_environment()
         self.workload.start()
 
+        # Clear statuses before resync as we want to update it anyway.
+        self.state.statuses.clear(scope="unit", component=self.name)
+
         # pbm has a flakely resync and it is necessary to wait for no actions to be running before
         # resync-ing. See: https://jira.percona.com/browse/PBM-1038
         for attempt in Retrying(
@@ -324,15 +384,14 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
             reraise=True,
         ):
             with attempt:
-                pbm_status = self.get_status()
-                # wait for backup/restore to finish
-                if isinstance(pbm_status, (MaintenanceStatus)):
-                    raise PBMBusyError
-
-                # if a resync is running restart the service
-                if isinstance(pbm_status, (WaitingStatus)):
-                    self.workload.restart()
-                    raise PBMBusyError
+                match self.backup_state():
+                    case BackupState.BACKUP_RUNNING | BackupState.RESTORE_RUNNING:
+                        raise PBMBusyError
+                    case BackupState.WAITING_TO_SYNC:
+                        self.workload.restart()
+                        raise PBMBusyError
+                    case _:
+                        continue
 
         # wait for re-sync and update charm status based on pbm syncing status. Need to wait for
         # 2 seconds for pbm_agent to receive the resync command before verifying.
@@ -371,7 +430,7 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
 
         return True
 
-    def set_config_options(self, credentials: dict[str, str]) -> None:
+    def set_config_options(self, credentials: dict) -> None:
         """Apply the configuration provided by S3 integrator.
 
         Args:
@@ -380,6 +439,7 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         # Add certificate to trust store
         if cert_chain_list := credentials.get("tls-ca-chain", None):
             self.dependent.save_ca_cert_to_trust_store(TrustStoreFiles.PBM, cert_chain_list)
+            self.share_certificate_with_shards(cert_chain_list)
             self.configure_and_restart(force=True)
 
         # Clear the current config file.
@@ -390,7 +450,9 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         for pbm_key, pbm_value in config.items():
             try:
                 self.workload.run_bin_command(
-                    "config", ["--set", f"{pbm_key}={pbm_value}"], environment=self.environment
+                    "config",
+                    ["--set", f"{pbm_key}={pbm_value}"],
+                    environment=self.environment,
                 )
             except WorkloadExecError:
                 logger.error(f"Failed to configure PBM option: {pbm_key}")
@@ -404,7 +466,9 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
                 "# this file is to be left empty. Changes in this file will be ignored.\n",
             )
         self.workload.run_bin_command(
-            "config", ["--file", str(self.workload.paths.pbm_config)], environment=self.environment
+            "config",
+            ["--file", str(self.workload.paths.pbm_config)],
+            environment=self.environment,
         )
 
     def retrieve_error_message(self, pbm_status: dict) -> str:
@@ -414,23 +478,32 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         Instead, it is in the log messages. pbm_agent also shows all the error messages for other
         replicas in the set. This method tries to handle both cases at once.
         """
-        try:
-            clusters = pbm_status["cluster"]
-            for cluster in clusters:
-                if cluster["rs"] == self.charm.app.name:
-                    break
+        app_name = self.charm.app.name
+        replica_info = (
+            f"{app_name}/{self.state.unit_peer_data.internal_address}:{MongoPorts.MONGODB_PORT}"
+        )
 
-            for host_info in cluster["nodes"]:
-                replica_info = (
-                    f"mongodb/{self.state.unit_peer_data.internal_address}:{MongoPorts.MONGOS_PORT}"
-                )
-                if host_info["host"] == replica_info:
-                    break
+        clusters = pbm_status.get("cluster")
 
-            return str(host_info["errors"])
-        except KeyError:
+        # No clusters means no error message
+        if not clusters:
             return ""
 
+        cluster: dict | None = next(
+            (_cluster for _cluster in clusters if _cluster.get("rs") == app_name), None
+        )
+
+        # No matching cluster means no error message
+        if not cluster:
+            return ""
+
+        for host_info in cluster.get("nodes", []):
+            if host_info.get("host") == replica_info:
+                return str(host_info.get("errors", ""))
+
+        # Default case, no error message
+        return ""
+
     def get_backup_error_status(self, backup_id: str) -> str:
         """Get the error status for a provided backup."""
         pbm_status = self.pbm_status
@@ -442,10 +515,10 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
 
         return ""
 
-    def process_pbm_error(self, pbm_status: str) -> str:
+    def process_pbm_error(self, pbm_status: str) -> BackupState | None:
         """Look up PBM status for errors."""
         error_message: str
-        message = ""
+
         try:
             pbm_as_dict = json.loads(pbm_status)
             error_message = self.retrieve_error_message(pbm_as_dict)
@@ -453,26 +526,39 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
             error_message = pbm_status
 
         if StatusCodeError.FORBIDDEN in error_message:
-            message = "s3 credentials are incorrect."
-        elif StatusCodeError.NOTFOUND in error_message:
-            message = "s3 configurations are incompatible."
-        elif StatusCodeError.MOVED_PERMANENTLY in error_message:
-            message = "s3 configurations are incompatible."
-        return message
+            return BackupState.INCORRECT_CREDS
+        if StatusCodeError.NOTFOUND in error_message:
+            return BackupState.INCOMPATIBLE_CONF
+        if StatusCodeError.MOVED_PERMANENTLY in error_message:
+            return BackupState.INCOMPATIBLE_CONF
 
-    def process_pbm_status(self, pbm_status: str) -> StatusBase:
+        if error_message:
+            logger.info("PBM error: %s", error_message)
+            return BackupState.UNKNOWN_ERROR
+
+        return None
+
+    def process_pbm_error_as_status(self, pbm_status: str) -> StatusObject | None:
+        """Processes the pbm error and returns it as an optional status object."""
+        if state := self.process_pbm_error(pbm_status):
+            return next(iter(self.map_backup_state_to_status(state)), None)
+        return None
+
+    def process_pbm_status(self, pbm_status: str) -> BackupState:
         """Processes the pbm status if there's no error."""
         pbm_as_dict: dict[str, dict] = json.loads(pbm_status)
         current_op = pbm_as_dict.get("running", {})
         match current_op:
             case {"type": "backup", "name": backup_id}:
-                return MaintenanceStatus(f"backup started/running, backup id: '{backup_id}'")
+                self.backup_id = backup_id
+                return BackupState.BACKUP_RUNNING
             case {"type": "restore", "name": backup_id}:
-                return MaintenanceStatus(f"restore started/running, backup id: '{backup_id}'")
+                self.backup_id = backup_id
+                return BackupState.RESTORE_RUNNING
             case {"type": "resync"}:
-                return WaitingStatus("waiting to sync s3 configurations.")
+                return BackupState.WAITING_TO_SYNC
             case _:
-                return ActiveStatus("")
+                return BackupState.ACTIVE
 
     def assert_can_restore(self, backup_id: str, remapping_pattern: str) -> None:
         """Does the status allow to restore.
@@ -481,16 +567,24 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
             check: boolean telling if the status allows to restore.
             reason: The reason if it is not possible to restore yet.
         """
-        pbm_status = self.get_status()
-        match pbm_status:
-            case MaintenanceStatus():
+        backup_state = self.backup_state()
+
+        match backup_state:
+            case BackupState.EMPTY:
+                return
+            case BackupState.BACKUP_RUNNING | BackupState.RESTORE_RUNNING:
                 raise InvalidPBMStatusError("Please wait for current backup/restore to finish.")
-            case WaitingStatus():
+            case BackupState.WAITING_TO_SYNC:
                 raise InvalidPBMStatusError(
-                    "Sync-ing configurations needs more time, must wait before listing backups."
+                    "Sync-ing configurations needs more time, must wait before restoring backups."
                 )
-            case BlockedStatus():
-                raise InvalidPBMStatusError(pbm_status.message)
+            case (
+                BackupState.MISSING_CONFIG
+                | BackupState.INCORRECT_CREDS
+                | BackupState.INCOMPATIBLE_CONF
+                | BackupState.UNKNOWN_ERROR
+            ):
+                raise InvalidPBMStatusError(self.map_backup_state_to_status(backup_state)[0])
             case _:
                 pass
 
@@ -503,31 +597,48 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
 
     def assert_can_backup(self) -> None:
         """Is PBM is a state where it can backup?"""
-        pbm_status = self.get_status()
-        match pbm_status:
-            case MaintenanceStatus():
+        backup_state = self.backup_state()
+
+        match backup_state:
+            case BackupState.EMPTY:
+                return
+            case BackupState.BACKUP_RUNNING | BackupState.RESTORE_RUNNING:
                 raise InvalidPBMStatusError(
                     "Can only create one backup at a time, please wait for current backup to finish."
                 )
-            case WaitingStatus():
+            case BackupState.WAITING_TO_SYNC:
                 raise InvalidPBMStatusError(
                     "Sync-ing configurations needs more time, must wait before creating backups."
                 )
-            case BlockedStatus():
-                raise InvalidPBMStatusError(pbm_status.message)
+            case (
+                BackupState.MISSING_CONFIG
+                | BackupState.INCORRECT_CREDS
+                | BackupState.INCOMPATIBLE_CONF
+                | BackupState.UNKNOWN_ERROR
+            ):
+                raise InvalidPBMStatusError(self.map_backup_state_to_status(backup_state)[0])
             case _:
                 return
 
     def assert_can_list_backup(self) -> None:
-        """Is PBM in a state to list backup?"""
-        pbm_status = self.get_status()
-        match pbm_status:
-            case WaitingStatus():
+        """Is PBM in a state to list backup?
+
+        Note: we permit this logic based on status since we aren't checking
+        `self.charm.unit.status`, instead `get_status` directly computes the status of pbm.
+        """
+        backup_state = self.backup_state()
+        match backup_state:
+            case BackupState.WAITING_TO_SYNC:
                 raise InvalidPBMStatusError(
                     "Sync-ing configurations needs more time, must wait before listing backups."
                 )
-            case BlockedStatus():
-                raise InvalidPBMStatusError(pbm_status.message)
+            case (
+                BackupState.MISSING_CONFIG
+                | BackupState.INCORRECT_CREDS
+                | BackupState.INCOMPATIBLE_CONF
+                | BackupState.UNKNOWN_ERROR
+            ):
+                raise InvalidPBMStatusError(self.map_backup_state_to_status(backup_state)[0])
             case _:
                 return
 
@@ -558,42 +669,63 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
             if current_pbm_op.get("type", "") == "resync":
                 # since this process takes several minutes we should let the user know
                 # immediately.
-                self.charm.status_manager.set_and_share_status(
-                    WaitingStatus("waiting to sync s3 configurations.")
+                self.charm.status_handler.set_running_status(
+                    BackupStatuses.PBM_WAITING_TO_SYNC.value,
+                    scope="unit",
+                    statuses_state=self.state.statuses,
+                    component_name=self.name,
                 )
                 raise ResyncError
         except WorkloadExecError as e:
-            self.charm.status_manager.set_and_share_status(
-                BlockedStatus(self.process_pbm_error(e.stdout))
-            )
+            if status := self.process_pbm_error_as_status(e.stdout):
+                self.state.statuses.add(status, scope="unit", component=self.name)
 
-    def _get_backup_restore_operation_result(
-        self, current_pbm_status: StatusBase, previous_pbm_status: StatusBase
-    ) -> str:
+    def _get_backup_restore_operation_result(self, current_pbm_status: BackupState) -> str | None:
         """Returns a string with the result of the backup/restore operation.
 
+        Note: current_pbm_status is a freshly calculated status from PBM directly, so we allow
+        calculations based on its result.
+
         The function call is expected to be only for not failed operations.
         The operation is taken from previous status of the unit and expected
         to contain the operation type (backup/restore) and the backup id.
+
+        This function does not work reliably for the following reasons.
+        1. Another status could get set and then the use of the previous_pbm_status is meaningless
+        2. If there was a PBM failure that wasn't noticed by a status check (i.e. in between hooks)
+        there is a chance that this function it will incorrectly report the backup/restore
+        succeeded.
+
+        TODO: Rework this and integrate it with COS - see DPE-6868 on JIRA for more info.
         """
-        if (
-            current_pbm_status.name == previous_pbm_status.name
-            and current_pbm_status.message == previous_pbm_status.message
-            and not isinstance(current_pbm_status, ActiveStatus)
-        ):
-            return f"Operation is still in progress: '{current_pbm_status.message}'"
+        previous_pbm_statuses = self.state.statuses.get(
+            scope="unit",
+            component=self.name,
+            running_status_only=True,
+            running_status_type="async",
+        ).root
+        # No previous PBM status, we can return.
+        if previous_pbm_statuses == []:
+            return None
 
-        if (
-            isinstance(previous_pbm_status, MaintenanceStatus)
-            and "backup id:" in previous_pbm_status.message
-        ):
-            backup_id = previous_pbm_status.message.split("backup id:")[-1].strip()
-            if "restore" in previous_pbm_status.message:
-                return f"Restore from backup {backup_id} completed successfully"
-            if "backup" in previous_pbm_status.message:
-                return f"Backup {backup_id} completed successfully"
+        previous_pbm_status = previous_pbm_statuses[0]
+        previous_operation = (
+            BackupState.BACKUP_RUNNING
+            if "Backup" in previous_pbm_status.message
+            else BackupState.RESTORE_RUNNING
+        )
+        previous_backup_id = previous_pbm_status.message.split("backup id:")[-1].strip()
 
-        return "Unknown operation result"
+        # Same operation and same backup ID.
+        if previous_operation == current_pbm_status and previous_backup_id == self.backup_id:
+            return f"Operation is still in progress: '{previous_pbm_status.message}'"
+
+        # Backup finished successfully.
+        if previous_operation == BackupState.BACKUP_RUNNING:
+            return f"Backup {previous_backup_id} completed successfully"
+
+        # Restore finished successfully.
+        return f"Restore from backup {previous_backup_id} completed successfully"
 
     def _is_backup_from_different_cluster(self, backup_status: str) -> bool:
         """Returns if a given backup was made on a different cluster."""
@@ -658,6 +790,24 @@ class BackupManager(Object, BackupConfigManager, StatusProvider):
         )
         return f"{current_cluster_name}={old_cluster_name}"
 
+    def share_certificate_with_shards(self, ca_chain_list: list[str]):
+        """Shares the certificates to shards if role is config-server."""
+        if self.state.is_role(MongoDBRoles.CONFIG_SERVER) and self.charm.unit.is_leader():
+            for relation in self.state.config_server_relation:
+                self.state.config_server_data_interface.update_relation_data(
+                    relation.id,
+                    {AppShardingComponentKeys.BACKUP_CA_SECRET.value: json.dumps(ca_chain_list)},
+                )
+
+    def remove_cert_from_shards(self):
+        """Removes the certificates from the shards databag."""
+        if self.state.is_role(MongoDBRoles.CONFIG_SERVER) and self.charm.unit.is_leader():
+            # Remove the certificate from all relations
+            for relation in self.state.config_server_relation:
+                self.state.config_server_data_interface.delete_relation_data(
+                    relation.id, [AppShardingComponentKeys.BACKUP_CA_SECRET.value]
+                )
+
 
 def map_s3_config_to_pbm_config(credentials: dict[str, str]):
     """Simple mapping from s3 integration to current status."""
