@@ -9,13 +9,14 @@ import json
 from logging import getLogger
 from typing import TYPE_CHECKING
 
-from ops import BlockedStatus
+from data_platform_helpers.advanced_statuses.models import StatusObject
 from ops.framework import Object
-from ops.model import Relation, StatusBase
+from ops.model import Relation
 from pymongo.errors import PyMongoError
 
 from single_kernel_mongo.config.literals import Scope, Substrates
 from single_kernel_mongo.config.relations import RelationNames
+from single_kernel_mongo.config.statuses import CharmStatuses, MongoDBStatuses, MongosStatuses
 from single_kernel_mongo.core.structured_config import MongoDBRoles
 from single_kernel_mongo.exceptions import (
     DeferrableError,
@@ -64,8 +65,10 @@ class ClusterProvider(Object):
             raise DeferrableFailedHookChecksError("DB is not initialised")
 
         if not self.is_valid_mongos_integration():
-            self.charm.status_manager.to_blocked(
-                "Relation to mongos not supported, config role must be config-server"
+            self.state.statuses.add(
+                MongoDBStatuses.UNSUPPORTED_MONGOS_REL.value,
+                scope="unit",
+                component=self.dependent.name,
             )
             raise NonDeferrableFailedHookChecksError(
                 "ClusterProvider is only executed by a config-server"
@@ -262,7 +265,11 @@ class ClusterRequirer(Object):
     def set_relation_created_status(self) -> None:
         """Just sets a status on relation created."""
         logger.info("Integrating to config-server")
-        self.charm.status_manager.to_waiting("Connecting to config-server")
+        self.state.statuses.set(
+            MongosStatuses.CONNECTING_TO_CONFIG_SERVER.value,
+            scope="unit",
+            component=self.dependent.name,
+        )
 
     def share_credentials_to_clients(self, username: str | None, password: str | None) -> None:
         """Database created event.
@@ -304,17 +311,25 @@ class ClusterRequirer(Object):
 
         if updated_keyfile or updated_config or not self.dependent.is_mongos_running():
             logger.info("Restarting mongos with new secrets.")
-            self.charm.status_manager.to_maintenance("starting mongos")
+            self.charm.status_handler.set_running_status(
+                MongosStatuses.STARTING_MONGOS.value, scope="unit"
+            )
+
             self.dependent.restart_charm_services()
 
             # Restart on highly loaded databases can be very slow (up to 10-20 minutes).
             if not self.dependent.is_mongos_running():
                 logger.info("Mongos has not started yet, deferring")
-                self.charm.status_manager.to_waiting("Waiting for mongos to start")
+                self.state.statuses.set(
+                    MongosStatuses.MONGOS_NOT_STARTED.value,
+                    scope="unit",
+                    component=self.dependent.name,
+                )
                 raise DeferrableError
 
-        self.charm.status_manager.to_active()
-
+        self.state.statuses.set(
+            CharmStatuses.ACTIVE_IDLE.value, scope="unit", component=self.dependent.name
+        )
         if self.charm.unit.is_leader():
             self.state.app_peer_data.db_initialised = True
             # In the K8S case, create the user
@@ -376,20 +391,28 @@ class ClusterRequirer(Object):
         if not self.charm.unit.is_leader():
             return
 
+        if not self.state.has_credentials():
+            # This happens in case of invalid integration, for example if it
+            # was integrated with a shard instead of a config-server
+            logger.info("No credentials found, not cleaning users.")
+            return
+
         # We are a Kubernetes Mongos Charm so we are in charge of our client
         # applications and their users and we proceed to remove the users we manage and their DBs.
         for relation in self.state.client_relations:
             self.dependent.mongo_manager.remove_user(relation)
             data_interface = DatabaseProviderData(self.model, relation.name)
             fields = data_interface.fetch_my_relation_data([relation.id])[relation.id]
-            data_interface.delete_relation_data(relation.id, list(fields.keys()))
-            secret_id = json.loads(
-                data_interface.fetch_relation_field(relation.id, "data") or "{}"
-            )["secret-user"]
 
-            user_secrets = self.charm.model.get_secret(id=secret_id)
-            user_secrets.remove_all_revisions()
-            user_secrets.get_content(refresh=True)
+            data_str = relation.data[next(iter(relation.units))].get("data", "{}")
+            secret_id = json.loads(data_str).get("secret-user")
+
+            data_interface.delete_relation_data(relation.id, list(fields.keys()))
+
+            if secret_id:
+                user_secrets = self.charm.model.get_secret(id=secret_id)
+                user_secrets.remove_all_revisions()
+                user_secrets.get_content(refresh=True)
             relation.data[self.charm.app].clear()
 
         # Also remove the local user.
@@ -429,19 +452,19 @@ class ClusterRequirer(Object):
 
         return False, False
 
-    def get_tls_statuses(self) -> StatusBase | None:
+    def get_tls_statuses(self) -> StatusObject | None:
         """Return statuses relevant to TLS."""
         mongos_has_tls, config_server_has_tls = self.tls_status()
         match (mongos_has_tls, config_server_has_tls):
             case False, True:
-                return BlockedStatus("mongos requires TLS to be enabled.")
+                return MongosStatuses.REQUIRES_TLS.value
             case True, False:
-                return BlockedStatus("mongos has TLS enabled, but config-server does not.")
+                return MongosStatuses.REQUIRES_NO_TLS.value
             case _:
                 pass
         if not self.is_ca_compatible():
             logger.error(
                 "mongos is integrated to a different CA than the config server. Please use the same CA for all cluster components."
             )
-            return BlockedStatus("mongos CA and Config-Server CA don't match.")
+            return MongosStatuses.CA_MISMATCH.value
         return None
