@@ -16,16 +16,25 @@ import logging
 from typing import TYPE_CHECKING
 
 from dacite import from_dict
+from data_platform_helpers.advanced_statuses.models import StatusObject
+from data_platform_helpers.advanced_statuses.protocol import ManagerStatusProtocol
+from data_platform_helpers.advanced_statuses.types import Scope
 from ops import Object
-from ops.model import ActiveStatus, BlockedStatus, Relation, StatusBase, WaitingStatus
-from pymongo.errors import AutoReconnect, PyMongoError, ServerSelectionTimeoutError
+from ops.model import Relation
+from pymongo.errors import (
+    AutoReconnect,
+    OperationFailure,
+    PyMongoError,
+    ServerSelectionTimeoutError,
+)
 
-from single_kernel_mongo.config.literals import Substrates
-from single_kernel_mongo.core.status_provider import StatusProvider
+from single_kernel_mongo.config.literals import MongoPorts, Substrates
+from single_kernel_mongo.config.statuses import CharmStatuses, MongodStatuses
 from single_kernel_mongo.core.structured_config import MongoDBRoles
 from single_kernel_mongo.exceptions import (
     DatabaseRequestedHasNotRunYetError,
     DeployedWithoutTrustError,
+    MissingCredentialsError,
     SetPasswordError,
 )
 from single_kernel_mongo.lib.charms.data_platform_libs.v0.data_interfaces import (
@@ -41,6 +50,7 @@ from single_kernel_mongo.utils.mongo_connection import MongoConnection, NotReady
 from single_kernel_mongo.utils.mongodb_users import (
     OPERATOR_ROLE,
     BackupUser,
+    LogRotateUser,
     MongoDBUser,
     MonitorUser,
     OperatorUser,
@@ -52,7 +62,7 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-class MongoManager(Object, StatusProvider):
+class MongoManager(Object, ManagerStatusProtocol):
     """Manager for Mongo related operations."""
 
     def __init__(
@@ -63,17 +73,23 @@ class MongoManager(Object, StatusProvider):
         substrate: Substrates,
     ) -> None:
         super().__init__(parent=dependent, key="managers")
+        self.name = "mongo"
         self.charm = dependent.charm
         self.workload = workload
         self.state = state
         self.substrate = substrate
+
         pod_name = self.model.unit.name.replace("/", "-")
         self.k8s = K8sManager(pod_name, self.model.name)
         if self.substrate == Substrates.K8S:
             try:
                 self.k8s.get_pod()
             except DeployedWithoutTrustError:
-                self.charm.status_manager.to_blocked("Charm deployed without `trust`")
+                self.state.statuses.add(
+                    CharmStatuses.DEPLOYED_WITHOUT_TRUST.value,
+                    scope="unit",
+                    component=self.charm.name,
+                )
 
     def mongod_ready(self, uri: str | None = None, direct: bool = True) -> bool:
         """Is MongoDB ready and running?
@@ -81,6 +97,8 @@ class MongoManager(Object, StatusProvider):
         Pass direct=True, when checking if a *single replica* is ready.
         Pass direct=False, when checking if the entire replica set is ready
         """
+        if not uri and self.state.is_role(MongoDBRoles.MONGOS):
+            uri = f"localhost:{MongoPorts.MONGOS_PORT}"
         actual_uri = uri or "localhost"
         with MongoConnection(EMPTY_CONFIGURATION, actual_uri, direct=direct) as direct_mongo:
             return direct_mongo.is_ready
@@ -114,6 +132,7 @@ class MongoManager(Object, StatusProvider):
         self.initialise_operator_user()
         self.initialise_user(MonitorUser)
         self.initialise_user(BackupUser)
+        self.initialise_user(LogRotateUser)
 
     def initialise_operator_user(self):
         """Creates initial admin user for MongoDB.
@@ -148,22 +167,26 @@ class MongoManager(Object, StatusProvider):
         if self.state.app_peer_data.is_user_created(user.username):
             return
         with MongoConnection(self.state.mongo_config) as mongo:
-            logger.debug(f"Creating the {user.username} user roles…")
+            logger.info(f"Creating the {user.username} user roles…")
             mongo.create_role(
                 role_name=user.mongodb_role,
                 privileges=user.privileges,
             )
-            logger.debug(f"Creating the {user.username} user...")
+            logger.info(f"Creating the {user.username} user...")
             config = self.state.mongodb_config_for_user(user)
             mongo.create_user(
                 config.username,
                 config.password,
                 config.supported_roles,
             )
+
         self.state.app_peer_data.set_user_created(user.username)
 
     def reconcile_mongo_users_and_dbs(
-        self, relation: Relation, relation_departing: bool = False, relation_changed: bool = False
+        self,
+        relation: Relation,
+        relation_departing: bool = False,
+        relation_changed: bool = False,
     ):
         """Oversees the users of the relation.
 
@@ -459,10 +482,12 @@ class MongoManager(Object, StatusProvider):
                     raise NotReadyError
                 mongo.add_replset_member(member)
 
-    def get_draining_shards(self, config: MongoConfiguration | None = None) -> list[str]:
+    def get_draining_shards(
+        self, shard_name: str, config: MongoConfiguration | None = None
+    ) -> list[str]:
         """Returns the shard that is currently draining."""
         with MongoConnection(config or self.state.mongos_config) as mongo:
-            draining_shards = mongo.get_draining_shards()
+            draining_shards = mongo.get_draining_shards(shard_name=shard_name)
 
             # in theory, this should always be a list of one. But if something has gone wrong we
             # should take note and log it
@@ -471,36 +496,73 @@ class MongoManager(Object, StatusProvider):
 
             return draining_shards
 
-    def get_status(self) -> StatusBase:
+    def get_statuses(self, scope: Scope, recompute: bool = False) -> list[StatusObject]:  # noqa: C901 (this function is complex but we can't reduce its complexity easily enough)
         """Generates the status of a unit based on its status reported by mongod."""
+        charm_statuses: list[StatusObject] = []
+
+        if not recompute:
+            return self.state.statuses.get(scope=scope, component=self.name).root
+
+        if not self.state.db_initialised:
+            return [MongodStatuses.WAITING_REPL_SET_INIT.value]
+
+        if scope == "app":
+            return []
+
         try:
             with MongoConnection(self.state.mongo_config) as mongo:
                 replset_status = mongo.get_replset_status()
 
-            unit_host = self.state.unit_peer_data.internal_address
-            if unit_host not in replset_status:
-                return WaitingStatus("Member being added.")
-
-            replica_status = replset_status[unit_host]
+            replica_status = replset_status.get(self.state.unit_peer_data.internal_address, "")
 
             match replica_status:
+                case "":
+                    return [MongodStatuses.MEMBER_BEING_ADDED.value]
                 case "PRIMARY":
-                    return ActiveStatus("Primary")
+                    charm_statuses.append(MongodStatuses.PRIMARY.value)
                 case "SECONDARY":
-                    return ActiveStatus("")
+                    charm_statuses.append(MongodStatuses.SECONDARY.value)
                 case "STARTUP" | "STARTUP2" | "ROLLBACK" | "RECOVERING":
-                    return WaitingStatus("Member is syncing...")
+                    return [MongodStatuses.MEMBER_SYNCING.value]
                 case "REMOVED":
-                    return WaitingStatus("Member is removing...")
+                    return [MongodStatuses.MEMBER_REMOVING.value]
                 case _:
-                    return BlockedStatus(replica_status)
+                    return [MongodStatuses.replset_status(replica_status)]
+
+            charm_statuses.extend(self.get_leader_statuses())
+            return charm_statuses
 
         except ServerSelectionTimeoutError as e:
             # Usually it is du to ReplicaSetNoPrimary
             logger.debug(f"Got error {e} while checking replica set status")
-            return WaitingStatus("Waiting for primary re-election.")
+            return [MongodStatuses.WAITING_ELECTION.value]
         except AutoReconnect as e:
             # AutoReconnect is raised when a connection to the database is lost and an attempt to
             # auto-reconnect will be made by pymongo.
             logger.debug("Got error: %s, while checking replica set status", str(e))
-            return WaitingStatus("Waiting to reconnect to unit...")
+            return [MongodStatuses.WAITING_RECONNECT.value]
+        except OperationFailure as e:
+            logger.warning("Authentication Failed: %s", e, exc_info=True)
+            return [MongodStatuses.WAITING_RECONFIG.value]
+        except MissingCredentialsError as e:
+            logger.warning("Missing credentials: %s", e, exc_info=True)
+            return [MongodStatuses.MISSING_CREDENTIALS]
+
+    def get_leader_statuses(self) -> list[StatusObject]:
+        """Returns statuses that juju leader can retrieve for mongo."""
+        charm_statuses: list[StatusObject] = []
+
+        if not self.charm.unit.is_leader():
+            return charm_statuses
+
+        try:
+            with MongoConnection(self.state.mongo_config) as mongo:
+                replset_members = mongo.get_replset_members()
+                config_hosts = mongo.config.hosts
+                if replset_members != config_hosts:
+                    return [MongodStatuses.WAITING_RECONFIG.value]
+        except PyMongoError as e:
+            logger.error("Error checking members, %s", e)
+            return [MongodStatuses.WAITING_RECONFIG.value]
+
+        return charm_statuses
