@@ -9,13 +9,16 @@ from __future__ import annotations
 import logging
 from typing import TYPE_CHECKING, final
 
+from data_platform_helpers.advanced_statuses.models import StatusObject
+from data_platform_helpers.advanced_statuses.protocol import ManagerStatusProtocol
+from data_platform_helpers.advanced_statuses.types import Scope as DPHScope
 from data_platform_helpers.version_check import (
     CrossAppVersionChecker,
     get_charm_revision,
 )
 from ops.framework import Object
-from ops.model import Container, MaintenanceStatus, Unit
-from pymongo.errors import PyMongoError, ServerSelectionTimeoutError
+from ops.model import Container, Unit
+from pymongo.errors import OperationFailure, PyMongoError, ServerSelectionTimeoutError
 from tenacity import Retrying, stop_after_attempt, wait_fixed
 from typing_extensions import override
 
@@ -28,21 +31,34 @@ from single_kernel_mongo.config.literals import (
     Substrates,
     UnitState,
 )
-from single_kernel_mongo.config.models import ROLES
+from single_kernel_mongo.config.models import ROLES, BackupState
 from single_kernel_mongo.config.relations import ExternalRequirerRelations, RelationNames
+from single_kernel_mongo.config.statuses import (
+    BackupStatuses,
+    CharmStatuses,
+    LdapStatuses,
+    MongoDBStatuses,
+    MongodStatuses,
+    ShardStatuses,
+)
 from single_kernel_mongo.core.kubernetes_upgrades import KubernetesUpgrade
 from single_kernel_mongo.core.machine_upgrades import MachineUpgrade
 from single_kernel_mongo.core.operator import OperatorProtocol
 from single_kernel_mongo.core.secrets import generate_secret_label
 from single_kernel_mongo.core.structured_config import MongoDBRoles
 from single_kernel_mongo.core.version_checker import VersionChecker
-from single_kernel_mongo.events.backups import INVALID_S3_INTEGRATION_STATUS, BackupEventsHandler
+from single_kernel_mongo.events.backups import (
+    BackupEventsHandler,
+)
 from single_kernel_mongo.events.cluster import ClusterConfigServerEventHandler
 from single_kernel_mongo.events.database import DatabaseEventsHandler
 from single_kernel_mongo.events.ldap import LDAPEventHandler
 from single_kernel_mongo.events.password_actions import PasswordActionEvents
 from single_kernel_mongo.events.primary_action import PrimaryActionHandler
-from single_kernel_mongo.events.sharding import ConfigServerEventHandler, ShardEventHandler
+from single_kernel_mongo.events.sharding import (
+    ConfigServerEventHandler,
+    ShardEventHandler,
+)
 from single_kernel_mongo.events.tls import TLSEventsHandler
 from single_kernel_mongo.events.upgrades import UpgradeEventHandler
 from single_kernel_mongo.exceptions import (
@@ -52,7 +68,9 @@ from single_kernel_mongo.exceptions import (
     InvalidLdapQueryTemplateError,
     InvalidLdapUserToDnMappingError,
     NonDeferrableFailedHookChecksError,
+    NotDrainedError,
     SetPasswordError,
+    ShardAuthError,
     ShardingMigrationError,
     UpgradeInProgressError,
     WaitingForLeaderError,
@@ -84,6 +102,7 @@ from single_kernel_mongo.utils.helpers import (
 from single_kernel_mongo.utils.mongo_connection import MongoConnection, NotReadyError
 from single_kernel_mongo.utils.mongodb_users import (
     BackupUser,
+    LogRotateUser,
     MonitorUser,
     OperatorUser,
     get_user_from_username,
@@ -105,7 +124,7 @@ logger = logging.getLogger(__name__)
 class MongoDBOperator(OperatorProtocol, Object):
     """Operator for MongoDB Related Charms."""
 
-    name = CharmKind.MONGOD
+    name = CharmKind.MONGOD.value
     workload: MongoDBWorkload
 
     def __init__(self, charm: AbstractMongoCharm):
@@ -250,10 +269,23 @@ class MongoDBOperator(OperatorProtocol, Object):
         )
         # END: Define config managers
 
+    @property
+    def components(self) -> tuple[ManagerStatusProtocol, ...]:
+        """The ordered list of components for this operator."""
+        return (
+            self,
+            self.mongo_manager,
+            self.shard_manager,
+            self.config_server_manager,
+            self.backup_manager,
+            self.ldap_manager,
+            self.upgrade_manager,
+        )
+
     # BEGIN: Handlers.
 
     @override
-    def on_install(self) -> None:
+    def install_workloads(self) -> None:
         """Handler on install."""
         if not self.workload.workload_present:
             raise ContainerNotReadyError
@@ -267,7 +299,7 @@ class MongoDBOperator(OperatorProtocol, Object):
         self.workload.write(self.workload.paths.config_file, "")
 
     @override
-    def on_start(self) -> None:
+    def prepare_for_startup(self) -> None:
         """Handler on start."""
         if not self.workload.workload_present:
             logger.debug("mongod installation is not ready yet.")
@@ -279,21 +311,19 @@ class MongoDBOperator(OperatorProtocol, Object):
 
         self._configure_workloads()
 
-        try:
-            logger.info("Starting MongoDB.")
-            self.charm.status_manager.to_maintenance("starting MongoDB")
-            self.start_charm_services()
-        except WorkloadServiceError as e:
-            logger.error(f"An exception occurred when starting mongod agent, error: {e}.")
-            self.charm.status_manager.to_blocked("couldn't start MongoDB")
-            return
+        logger.info("Starting MongoDB.")
+        self.charm.status_handler.set_running_status(
+            MongoDBStatuses.STARTING_MONGODB.value, scope="unit"
+        )
 
-        # Open ports:
-        try:
-            self.open_ports()
-        except WorkloadExecError:
-            self.charm.status_manager.to_blocked("failed to open TCP port for MongoDB")
-            raise
+        for attempt in Retrying(
+            stop=stop_after_attempt(5),
+            wait=wait_fixed(5),
+            reraise=True,
+        ):
+            with attempt:
+                self.start_charm_services()
+                self.open_ports()
 
         # This seems unnecessary
         # if self.substrate == Substrates.K8S:
@@ -302,16 +332,17 @@ class MongoDBOperator(OperatorProtocol, Object):
         #        raise WorkloadNotReadyError
 
         if not self.mongo_manager.mongod_ready():
-            self.charm.status_manager.to_waiting("waiting for MongoDB to start")
             raise WorkloadNotReadyError
 
-        self.charm.status_manager.to_active(None)
+        self.state.statuses.set(CharmStatuses.ACTIVE_IDLE.value, scope="unit", component=self.name)
 
         try:
             self._initialise_replica_set()
         except (NotReadyError, PyMongoError, WorkloadExecError) as e:
             logger.error(f"Deferring on start: error={e}")
-            self.charm.status_manager.to_waiting("waiting to initialize replica set")
+            self.state.statuses.add(
+                MongodStatuses.WAITING_REPL_SET_INIT.value, scope="unit", component=self.name
+            )
             raise
 
         try:
@@ -320,7 +351,7 @@ class MongoDBOperator(OperatorProtocol, Object):
             logger.error("Could not restart the related services.")
             return
 
-        self.charm.status_manager.to_active()
+        self.state.statuses.set(CharmStatuses.ACTIVE_IDLE.value, scope="unit", component=self.name)
 
         if self.substrate == Substrates.K8S:
             # K8S upgrades result in the start hook getting fired following this pattern
@@ -328,7 +359,7 @@ class MongoDBOperator(OperatorProtocol, Object):
             self.upgrade_manager._reconcile_upgrade()
 
     @override
-    def on_stop(self) -> None:  # pragma: nocover
+    def prepare_for_shutdown(self) -> None:  # pragma: nocover
         """Handler for the stop event.
 
         K8S Only, VM has nothing to do on this step.
@@ -372,7 +403,7 @@ class MongoDBOperator(OperatorProtocol, Object):
             return
 
     @override
-    def on_config_changed(self) -> None:
+    def update_config_and_restart(self) -> None:
         """Listen to changes in application configuration.
 
         To prevent a user from migrating a cluster, and causing the component to become
@@ -429,7 +460,7 @@ class MongoDBOperator(OperatorProtocol, Object):
         self.ldap_events.restart_if_ready_event.emit()
 
     @override
-    def on_leader_elected(self) -> None:
+    def new_leader(self) -> None:
         """Handles the leader elected event.
 
         Generates the keyfile and users credentials.
@@ -438,12 +469,12 @@ class MongoDBOperator(OperatorProtocol, Object):
             self.state.set_keyfile(self.workload.generate_keyfile())
 
         # Sets the password for the system users
-        for user in (OperatorUser, BackupUser, MonitorUser):
+        for user in (OperatorUser, BackupUser, MonitorUser, LogRotateUser):
             if not self.state.get_user_password(user):
                 self.state.set_user_password(user, self.workload.generate_password())
 
     @override
-    def on_relation_joined(self) -> None:
+    def new_peer(self) -> None:
         """Handle relation joined events.
 
         In this event, we first check for status checks (are we leader, is the
@@ -458,10 +489,10 @@ class MongoDBOperator(OperatorProtocol, Object):
             )
             raise UpgradeInProgressError
 
-        self.on_relation_changed()
+        self.peer_changed()
         self.update_related_hosts()
 
-    def on_relation_changed(self) -> None:
+    def peer_changed(self) -> None:
         """Handle relation changed events.
 
         Adds the unit as a replica to the MongoDB replica set.
@@ -474,8 +505,9 @@ class MongoDBOperator(OperatorProtocol, Object):
         # units receiving a relation changed event. We must update the monitor
         # and pbm URI if the password changes so that COS/pbm can continue to
         # work.
-        self.mongodb_exporter_config_manager.configure_and_restart()
-        self.backup_manager.configure_and_restart()
+        if self.state.db_initialised and self.workload.active():
+            self.mongodb_exporter_config_manager.configure_and_restart()
+            self.backup_manager.configure_and_restart()
 
         # only leader should configure replica set and we should do it only if
         # the replica set is initialised.
@@ -493,11 +525,13 @@ class MongoDBOperator(OperatorProtocol, Object):
             self.mongo_manager.process_added_units()
         except (NotReadyError, PyMongoError) as e:
             logger.error(f"Not reconfiguring: error={e}")
-            self.charm.status_manager.to_waiting("waiting to reconfigure replica set")
+            self.state.statuses.add(
+                MongodStatuses.WAITING_RECONFIG.value, scope="unit", component=self.name
+            )
             raise
 
     @override
-    def on_secret_changed(self, secret_label: str, secret_id: str) -> None:
+    def update_secrets_and_restart(self, secret_label: str, secret_id: str) -> None:
         """Handles secrets changes event.
 
         When user run set-password action, juju leader changes the password inside the database
@@ -518,13 +552,14 @@ class MongoDBOperator(OperatorProtocol, Object):
         # Always update the PBM and mongodb exporter configuration so that if
         # the secret changed, the configuration is updated and will still work
         # afterwards.
-        self.mongodb_exporter_config_manager.configure_and_restart()
-        self.backup_manager.configure_and_restart()
+        if self.workload.active():
+            self.mongodb_exporter_config_manager.configure_and_restart()
+            self.backup_manager.configure_and_restart()
 
         # Always process the statuses.
 
     @override
-    def on_relation_departed(self, departing_unit: Unit | None) -> None:
+    def peer_leaving(self, departing_unit: Unit | None) -> None:
         """Handles the relation departed events."""
         if not self.charm.unit.is_leader() or departing_unit == self.charm.unit:
             return
@@ -536,10 +571,9 @@ class MongoDBOperator(OperatorProtocol, Object):
                 "Removing replicas during an upgrade is not supported. The charm may be in a broken, unrecoverable state"
             )
         self.update_hosts()
-        self.charm.status_manager.process_and_share_statuses()
 
     @override
-    def on_storage_attached(self) -> None:  # pragma: nocover
+    def prepare_storage(self) -> None:  # pragma: nocover
         """Handler for `storage_attached` event.
 
         This should handle fixing the permissions for the data dir.
@@ -558,7 +592,7 @@ class MongoDBOperator(OperatorProtocol, Object):
         )
 
     @override
-    def on_storage_detaching(self) -> None:
+    def prepare_storage_for_shutdown(self) -> None:
         """Before storage detaches, allow removing unit to remove itself from the set.
 
         If the removing unit is primary also allow it to step down and elect another unit as
@@ -581,7 +615,9 @@ class MongoDBOperator(OperatorProtocol, Object):
                 raise EarlyRemovalOfConfigServerError(early_removal_message)
             if self.state.is_role(MongoDBRoles.SHARD) and self.state.shard_relation is not None:
                 logger.info("Wait for shard to drain before detaching storage.")
-                self.charm.status_manager.to_maintenance("Draining shard from cluster")
+                self.charm.status_handler.set_running_status(
+                    ShardStatuses.DRAINING_SHARD.value, scope="unit"
+                )
                 mongos_hosts = self.state.shard_state.mongos_hosts
                 self.shard_manager.wait_for_draining(mongos_hosts)
                 logger.info("Shard successfully drained storage.")
@@ -590,7 +626,10 @@ class MongoDBOperator(OperatorProtocol, Object):
         try:
             # retries over a period of 10 minutes in an attempt to resolve race conditions it is
             # not possible to defer in storage detached.
-            logger.debug("Removing %s from replica set", self.state.unit_peer_data.internal_address)
+            logger.debug(
+                "Removing %s from replica set",
+                self.state.unit_peer_data.internal_address,
+            )
             for attempt in Retrying(
                 stop=stop_after_attempt(600),
                 wait=wait_fixed(1),
@@ -605,22 +644,29 @@ class MongoDBOperator(OperatorProtocol, Object):
                 self.charm.unit.name,
             )
         except PyMongoError as e:
-            logger.error("Failed to remove %s from replica set, error=%r", self.charm.unit.name, e)
+            logger.error(
+                "Failed to remove %s from replica set, error=%r",
+                self.charm.unit.name,
+                e,
+            )
 
     @override
-    def on_update_status(self) -> None:
+    def update_status(self) -> None:
         """Status update Handler."""
-        if not self.pass_status_basic_checks():
+        # TODO update the usage of this once the spec is approved and we have a consistent way of
+        # handling statuses
+        if self.basic_statuses():
+            logger.info("Early return invalid statuses.")
             return
 
         if self.state.is_role(MongoDBRoles.SHARD):
             shard_has_tls, config_server_has_tls = self.shard_manager.tls_status()
             if config_server_has_tls and not shard_has_tls:
-                self.charm.status_manager.to_blocked("Shard requires TLS to be enabled")
+                logger.info("Shard is missing TLS.")
                 return
 
         if not self.mongo_manager.mongod_ready():
-            self.charm.status_manager.to_waiting("Waiting for MongoDB to start")
+            logger.info("Mongod not ready.")
             return
 
         if self.substrate == Substrates.K8S:
@@ -632,12 +678,14 @@ class MongoDBOperator(OperatorProtocol, Object):
         if not self.state.upgrade_in_progress:
             try:
                 self.perform_self_healing()
-            except ServerSelectionTimeoutError as e:
+            except (ServerSelectionTimeoutError, OperationFailure) as e:
                 logger.warning(f"Failed to perform self healing: {e}")
+            except ShardAuthError:
+                logger.warning("Failed to add shard")
+            except NotDrainedError:
+                logger.warning("Still draining shard.")
 
-        self.charm.status_manager.process_and_share_statuses()
-
-    def on_set_password_action(self, username: str, password: str | None = None) -> tuple[str, str]:
+    def set_password(self, username: str, password: str | None = None) -> tuple[str, str]:
         """Handler for the set password action."""
         self.assert_pass_password_checks()
 
@@ -655,19 +703,17 @@ class MongoDBOperator(OperatorProtocol, Object):
         if user == MonitorUser:
             # Update and restart mongodb exporter.
             self.mongodb_exporter_config_manager.configure_and_restart()
+        if user == LogRotateUser:
+            # Update and restart logrotate.
+            self.logrotate_config_manager.configure_and_restart()
         if user in (OperatorUser, BackupUser) and self.state.is_role(MongoDBRoles.CONFIG_SERVER):
             self.config_server_manager.update_credentials(
                 user.password_key_name,
                 new_password,
             )
 
-        self.charm.status_manager.process_and_share_statuses()
         return new_password, secret_id
 
-    def on_get_password_action(self, username: str) -> str:
-        """Handler for the get password action."""
-        return self.get_password(username)
-
     # END: Handlers.
 
     def assert_pass_password_checks(self) -> None:
@@ -684,7 +730,8 @@ class MongoDBOperator(OperatorProtocol, Object):
             raise NonDeferrableFailedHookChecksError(
                 "Cannot set password on shard, please set password on config-server."
             )
-        if isinstance(self.backup_manager.get_status(), MaintenanceStatus):
+        pbm_status = self.backup_manager.backup_state()
+        if pbm_status in (BackupState.BACKUP_RUNNING, BackupState.RESTORE_RUNNING):
             raise NonDeferrableFailedHookChecksError(
                 "Cannot change a password while a backup/restore is in progress."
             )
@@ -702,8 +749,9 @@ class MongoDBOperator(OperatorProtocol, Object):
         receive a relation event.
         """
         # All nodes should restart PBM and MongoDBExporter if it's not running
-        self.mongodb_exporter_config_manager.configure_and_restart()
-        self.backup_manager.configure_and_restart()
+        if self.workload.active():
+            self.mongodb_exporter_config_manager.configure_and_restart()
+            self.backup_manager.configure_and_restart()
 
         if not self.charm.unit.is_leader():
             logger.debug("Only the leader can perform reconfigurations to the replica set.")
@@ -713,7 +761,7 @@ class MongoDBOperator(OperatorProtocol, Object):
         self.update_hosts()
         # Add in any new IPs to the replica set. Relation handlers require a reference to
         # a unit.
-        self.on_relation_changed()
+        self.peer_changed()
 
         # make sure all nodes in the replica set have the same priority for re-election. This is
         # necessary in the case that pre-upgrade hook fails to reset the priority of election for
@@ -816,47 +864,38 @@ class MongoDBOperator(OperatorProtocol, Object):
 
         If we are running as config-server, we should update both mongod and mongos environments.
         """
-        self.config_manager.configure_and_restart(force=force)
-        if self.state.is_role(MongoDBRoles.CONFIG_SERVER):
-            self.mongos_config_manager.configure_and_restart(force=force)
+        try:
+            self.config_manager.configure_and_restart(force=force)
+            if self.state.is_role(MongoDBRoles.CONFIG_SERVER):
+                self.mongos_config_manager.configure_and_restart(force=force)
+        except WorkloadServiceError as e:
+            logger.error("An exception occurred when starting mongod agent, error: %s.", str(e))
+            self.charm.state.statuses.add(
+                MongoDBStatuses.WAITING_FOR_MONGODB_START.value, scope="unit", component=self.name
+            )
+            raise
 
     def _restart_related_services(self) -> None:
         """Restarts mongodb exporter and backup manager."""
         try:
             self.mongodb_exporter_config_manager.configure_and_restart()
         except WorkloadServiceError:
-            self.charm.status_manager.to_blocked("couldn't start mongodb exporter")
+            self.state.statuses.add(
+                MongoDBStatuses.WAITING_FOR_EXPORTER_START.value,
+                scope="unit",
+                component=self.name,
+            )
             raise
 
         try:
             self.backup_manager.configure_and_restart()
         except WorkloadServiceError:
-            self.charm.status_manager.to_blocked("couldn't start pbm-agent")
+            self.state.statuses.add(
+                BackupStatuses.WAITING_FOR_PBM_START.value, scope="unit", component=self.name
+            )
             raise
 
-    def pass_status_basic_checks(self) -> bool:
-        """Integration and initial checks for update-status events."""
-        if not self.state.is_sharding_component and self.state.has_sharding_integration:
-            self.charm.status_manager.to_blocked("sharding interface cannot be used by replicas")
-            return False
-        if not self.backup_manager.is_valid_s3_integration():
-            self.charm.status_manager.to_blocked(INVALID_S3_INTEGRATION_STATUS)
-            return False
-        if (
-            revision_mismatch_status
-            := self.cluster_version_checker.get_cluster_mismatched_revision_status()
-        ):
-            self.charm.status_manager.set_and_share_status(revision_mismatch_status)
-            return False
-        if not self.cluster_manager.is_valid_mongos_integration():
-            self.charm.status_manager.to_blocked(
-                "Relation to mongos not supported, config role must be config-server"
-            )
-            return False
-        if not self.state.db_initialised:
-            return False
-
-        return True
+        self.logrotate_config_manager.configure_and_restart()
 
     @override
     def is_relation_feasible(self, rel_name: str) -> bool:
@@ -872,8 +911,9 @@ class MongoDBOperator(OperatorProtocol, Object):
                 self.state.app_peer_data.role,
                 rel_name,
             )
-            self.charm.status_manager.to_blocked(
-                "Sharding roles do not support database interface."
+
+            self.state.statuses.add(
+                MongoDBStatuses.INVALID_DB_REL_ON_SHARD.value, scope="unit", component=self.name
             )
             return False
         if not self.state.is_sharding_component and rel_name == RelationNames.SHARDING:
@@ -882,7 +922,9 @@ class MongoDBOperator(OperatorProtocol, Object):
                 self.state.app_peer_data.role,
                 rel_name,
             )
-            self.charm.status_manager.to_blocked("sharding interface cannot be used by replicas")
+            self.state.statuses.add(
+                MongoDBStatuses.SHARDING_ON_REPLICA.value, scope="unit", component=self.name
+            )
             return False
         return True
 
@@ -892,9 +934,6 @@ class MongoDBOperator(OperatorProtocol, Object):
         self.config_manager.set_environment()
         self.mongos_config_manager.set_environment()
 
-        # Start logrotate
-        self.logrotate_config_manager.configure_and_restart()
-
         # Instantiate the keyfile
         self.instantiate_keyfile()
 
@@ -949,3 +988,62 @@ class MongoDBOperator(OperatorProtocol, Object):
     def is_removing_last_replica(self) -> bool:
         """Returns True if the last replica (juju unit) is getting removed."""
         return self.state.planned_units == 0 and len(self.state.peers_units) == 0
+
+    def basic_statuses(self) -> list[StatusObject]:
+        """Basic checks."""
+        statuses = []
+        if not self.cluster_manager.is_valid_mongos_integration():
+            statuses.append(MongoDBStatuses.UNSUPPORTED_MONGOS_REL.value)
+
+        if not self.backup_manager.is_valid_s3_integration():
+            statuses.append(MongoDBStatuses.INVALID_S3_INTEGRATION_STATUS.value)
+
+        if self.state.is_role(MongoDBRoles.REPLICATION) and (
+            self.state.config_server_relation or self.state.shard_relation
+        ):
+            statuses.append(MongoDBStatuses.SHARDING_ON_REPLICA.value)
+
+        if self.state.client_relations and self.state.is_sharding_component:
+            statuses.append(MongoDBStatuses.INVALID_DB_REL_ON_SHARD.value)
+
+        if not self.state.is_sharding_component and self.state.has_sharding_integration:
+            statuses.append(MongoDBStatuses.SHARDING_ON_REPLICA.value)
+        elif rev_status := self.cluster_version_checker.get_cluster_mismatched_revision_status():
+            # don't bother checking revision mismatch on sharding interface if replica
+            statuses.append(rev_status)
+
+        return statuses
+
+    def get_statuses(self, scope: DPHScope, recompute: bool = False) -> list[StatusObject]:  # noqa: C901 # We know, this function is complex.
+        """Returns the statuses of the charm manager."""
+        charm_statuses: list[StatusObject] = []
+
+        if not recompute:
+            return self.state.statuses.get(scope=scope, component=self.name).root
+
+        if scope == "app":
+            return charm_statuses
+
+        if not is_valid_ldapusertodnmapping(self.config.ldap_user_to_dn_mapping):
+            logger.error("Invalid LDAP Config - Please refer to the config option description.")
+            charm_statuses.append(LdapStatuses.INVALID_LDAP_USER_MAPPING.value)
+        if not is_valid_ldap_options(
+            self.config.ldap_user_to_dn_mapping, self.config.ldap_query_template
+        ):
+            logger.info("Invalid LDAP Config - Please refer to the config option description.")
+            charm_statuses.append(LdapStatuses.INVALID_LDAP_QUERY_TEMPLATE.value)
+
+        if not self.workload.workload_present:
+            return [CharmStatuses.MONGODB_NOT_INSTALLED.value]
+
+        charm_statuses += self.basic_statuses()
+
+        if not self.state.db_initialised:
+            charm_statuses.append(MongoDBStatuses.WAITING_FOR_MONGODB_START.value)
+
+        if not self.mongodb_exporter_config_manager.workload.active():
+            charm_statuses.append(MongoDBStatuses.WAITING_FOR_EXPORTER_START.value)
+
+            return charm_statuses
+
+        return charm_statuses
