@@ -12,8 +12,8 @@ from ipaddress import IPv4Address, IPv6Address
 from typing import TYPE_CHECKING, TypeVar
 from urllib.parse import quote
 
+from data_platform_helpers.advanced_statuses.protocol import StatusesState, StatusesStateProtocol
 from ops import Object, Relation, Unit
-from ops.model import ActiveStatus, BlockedStatus, StatusBase
 from pymongo.errors import OperationFailure, ServerSelectionTimeoutError
 
 from single_kernel_mongo.config.literals import (
@@ -37,6 +37,7 @@ from single_kernel_mongo.core.structured_config import (
     MongoDBRoles,
 )
 from single_kernel_mongo.core.workload import MongoPaths
+from single_kernel_mongo.exceptions import MissingCredentialsError
 from single_kernel_mongo.lib.charms.data_platform_libs.v0.data_interfaces import (
     DatabaseProviderData,
     DatabaseRequirerData,
@@ -61,13 +62,20 @@ from single_kernel_mongo.state.tls_state import TLSState
 from single_kernel_mongo.state.unit_peer_state import (
     UnitPeerReplicaSet,
 )
-from single_kernel_mongo.state.upgrade_state import AppUpgradePeerData, UnitUpgradePeerData
-from single_kernel_mongo.utils.helpers import generate_relation_departed_key, unit_number
+from single_kernel_mongo.state.upgrade_state import (
+    AppUpgradePeerData,
+    UnitUpgradePeerData,
+)
+from single_kernel_mongo.utils.helpers import (
+    generate_relation_departed_key,
+    unit_number,
+)
 from single_kernel_mongo.utils.mongo_config import MongoConfiguration
 from single_kernel_mongo.utils.mongo_connection import MongoConnection
 from single_kernel_mongo.utils.mongo_error_codes import MongoErrorCodes
 from single_kernel_mongo.utils.mongodb_users import (
     BackupUser,
+    LogRotateUser,
     MongoDBUser,
     MonitorUser,
     OperatorUser,
@@ -84,7 +92,7 @@ if TYPE_CHECKING:
 logger = logging.getLogger()
 
 
-class CharmState(Object):
+class CharmState(Object, StatusesStateProtocol):
     """The Charm State object.
 
     This object represents the charm state, including the different relations
@@ -99,7 +107,10 @@ class CharmState(Object):
     """
 
     def __init__(
-        self, charm: AbstractMongoCharm[T, U], substrate: Substrates, charm_role: CharmSpec
+        self,
+        charm: AbstractMongoCharm[T, U],
+        substrate: Substrates,
+        charm_role: CharmSpec,
     ):
         super().__init__(parent=charm, key="charm_state")
         self.charm_role = charm_role
@@ -108,6 +119,9 @@ class CharmState(Object):
         self.secrets = SecretCache(charm)
         self.peer_relation_name = charm.peer_rel_name.value
         self.ldap_peer_relation_name = PeerRelationNames.LDAP_PEERS.value
+        self.statuses_relation_name = PeerRelationNames.STATUS_PEERS.value
+
+        self.statuses = StatusesState(self, self.statuses_relation_name)
 
         self.peer_app_interface = DataPeerData(
             self.model,
@@ -523,7 +537,9 @@ class CharmState(Object):
         """The cluster peer relation."""
         return {
             unit: DataPeerOtherUnitData(
-                model=self.model, unit=unit, relation_name=RelationNames.UPGRADE_VERSION.value
+                model=self.model,
+                unit=unit,
+                relation_name=RelationNames.UPGRADE_VERSION.value,
             )
             for unit in self.peers_units
         }
@@ -684,30 +700,6 @@ class CharmState(Object):
         self.unit_peer_data.update({rel_departed_key: json.dumps(scaling_down)})
         return scaling_down
 
-    def share_status_with_config_server(self, status: StatusBase) -> None:
-        """Shares this shard's status with the config server.
-
-        This is primarily useful for the cluster upgrades, since the
-        config-server will need to ensure all units are healthy.
-        """
-        if not self.is_role(MongoDBRoles.SHARD):
-            return
-        if not self.shard_relation:
-            return
-
-        # All following cases write in the databag shared with the config server.
-        if isinstance(status, ActiveStatus):
-            self.unit_shard_state.status_ready_for_upgrade = True
-            return
-        if not isinstance(status, BlockedStatus):
-            self.unit_shard_state.status_ready_for_upgrade = False
-            return
-        if status.message and "is not up-to date with config-server" in status.message:
-            self.unit_shard_state.status_ready_for_upgrade = True
-            return
-
-        self.unit_shard_state.status_ready_for_upgrade = False
-
     @property
     def upgrade_resumed(self) -> bool:
         """Whether the user has resumed upgrade with juju action."""
@@ -742,6 +734,14 @@ class CharmState(Object):
 
     # BEGIN: Configuration accessors
 
+    def has_credentials(self) -> bool:
+        """Checks if we have received credentials or not."""
+        try:
+            self.mongo_config
+            return True
+        except MissingCredentialsError:
+            return False
+
     def mongodb_config_for_user(
         self,
         user: MongoDBUser,
@@ -812,6 +812,11 @@ class CharmState(Object):
         """Mongo Configuration for the monitoring user."""
         return self.mongodb_config_for_user(MonitorUser, hosts=self.internal_hosts)
 
+    @property
+    def logrotate_config(self) -> MongoConfiguration:
+        """Mongo Configuration for the logrotate user."""
+        return self.mongodb_config_for_user(LogRotateUser, standalone=True)
+
     @property
     def operator_config(self) -> MongoConfiguration:
         """Mongo Configuration for the operator user."""
@@ -838,7 +843,7 @@ class CharmState(Object):
         ):
             port = None
         if not username or not password:
-            raise Exception("Missing credentials.")
+            raise MissingCredentialsError("Missing credentials.")
 
         return MongoConfiguration(
             database=database,
