@@ -26,6 +26,7 @@ from single_kernel_mongo.exceptions import (
     NotDrainedError,
     NotEnoughSpaceError,
     ShardNotInClusterError,
+    ShardNotPlannedForRemovalError,
 )
 from single_kernel_mongo.utils.helpers import hostname_from_hostport, hostname_from_shardname
 from single_kernel_mongo.utils.mongo_config import MongoConfiguration
@@ -146,13 +147,20 @@ class MongoConnection:
 
         Grant read and write privileges for specified database.
         """
-        self.client.admin.command(
-            "createUser",
-            value=username,
-            pwd=password,
-            roles=roles,
-            mechanisms=["SCRAM-SHA-256"],
-        )
+        try:
+            self.client.admin.command(
+                "createUser",
+                value=username,
+                pwd=password,
+                roles=roles,
+                mechanisms=["SCRAM-SHA-256"],
+            )
+        except OperationFailure as e:
+            if e.code == MongoErrorCodes.USER_ALREADY_EXISTS:
+                logger.info("Role already exists")
+                return
+            logger.error("Cannot add user. error=%r", e)
+            raise
 
     def update_user(self, config: MongoConfiguration):
         """Update grants on database."""
@@ -387,20 +395,33 @@ class MongoConnection:
         shard_list = self.client.admin.command("listShards")
         return {hostname_from_shardname(member["host"]) for member in shard_list["shards"]}
 
-    def add_shard(self, shard_name, shard_hosts, shard_port=MongoPorts.MONGODB_PORT):
+    def add_shard(
+        self, shard_name: str, shard_hosts: list[str], shard_port=MongoPorts.MONGODB_PORT
+    ):
         """Adds shard to the cluster.
 
         Raises:
             ConfigurationError, OperationFailure
         """
-        shard_hosts = ",".join(f"{host}:{shard_port}" for host in shard_hosts)
-        shard_url = f"{shard_name}/{shard_hosts}"
+        shard_hosts = [f"{host}:{shard_port}" for host in shard_hosts]
         if shard_name in self.get_shard_members():
             logger.info("Skipping adding shard %s, shard is already in cluster", shard_name)
             return
 
         logger.info("Adding shard %s", shard_name)
-        self.client.admin.command("addShard", shard_url)
+        # We can use only one host of the shard and we don't know which ones
+        # are added yet to the replica set so we loop over it.
+        for shard_host in shard_hosts:
+            try:
+                shard_url = f"{shard_name}/{shard_host}"
+                self.client.admin.command("addShard", shard_url)
+                return
+            except OperationFailure as err:
+                # This means that the shard host is not yet in the shard replica set.
+                # Other errors should be raised as they indicate something else.
+                if err.code == MongoErrorCodes.OPERATION_FAILED:
+                    continue
+                raise
 
     def pre_remove_shard_checks(self, shard_name: str) -> None:
         """Performs a series of checks for removing a shard from the cluster.
@@ -664,11 +685,14 @@ class MongoConnection:
 
         return (candidate_shard, candidate_free_space)
 
-    def get_draining_shards(self) -> list[str]:
+    def get_draining_shards(self, shard_name: str) -> list[str]:
         """Returns a list of the shards currently draining."""
         sc_status = self.client.admin.command("listShards")
         draining_shards = []
         for shard in sc_status["shards"]:
+            if shard["_id"] == shard_name and "draining" not in shard:
+                raise ShardNotPlannedForRemovalError
+
             if shard.get("draining", False):
                 draining_shards.append(shard["_id"])
 
